unit PatrickLampke.Hashing.Types;

{$mode objfpc}{$H+}
{$modeswitch typehelpers}

interface

uses
  Classes, SysUtils, strutils;

type
  TSHA256 = packed array[0..31] of Byte;
  PSHA256 = ^TSHA256;

  { TSHA256Helper }

  TSHA256Helper = type helper for TSHA256
  	function ToString: String;
  end;

  function IsSHA256(const s: String): Boolean;
  function SHA256ToString(const Hash: TSHA256): String;
  function StringToSHA256(const s: String): TSHA256;

implementation

function IsSHA256(const s: String): Boolean;
const
  Allowed = ['0'..'9', 'a'..'f', 'A'..'F'];
var
  i, l: Integer;
begin
	l := Length(s);
  if l = 64 then begin
    Result := true;
	  for i := 1 to l do begin
      if not (s[i] in Allowed) then begin
			  Result := false;
        Break;
      end;
    end;
  end
  else Result := false;
end;

function SHA256ToString(const Hash: TSHA256): String;
var
  i: Integer;
  s: String = '';
begin
  for i := 0 to 31 do begin
		s += IntToHex(Hash[i], 2);
  end;
  Result := s;
end;

function StringToSHA256(const s: String): TSHA256;
var
  i: Integer;
  Current: String;
  Hash: TSHA256;
begin
	if IsSHA256(s) then begin
		for i := 0 to 31 do begin
      try
        Current := Copy(s, (i * 2) + 1, 2);
    	  Hash[i] := Hex2Dec(Current);
      except
        // nothing
      end;
    end;
  end;
  Result := Hash;
end;

{ TSHA256Helper }

function TSHA256Helper.ToString: String;
begin
  Result := SHA256ToString(Self);
end;

end.

